@@ -1,7 +1,7 @@
 use std::is_x86_feature_detected;
 
 /// Computes hamming distance (naive version)
-pub(crate) fn distance_naive(x: &[u8], y: &[u8]) -> u64 {
+pub fn distance_naive(x: &[u8], y: &[u8]) -> u64 {
     assert_eq!(x.len(), y.len());
     let mut accum: u64 = 0;
     for i in 0..x.len() {
@@ -11,10 +11,20 @@ pub(crate) fn distance_naive(x: &[u8], y: &[u8]) -> u64 {
     accum
 }
 
+/// computes hamming weight (naive version)
+pub fn weight_naive(x: &[u8]) -> u64 {
+    let mut accum: u64 = 0;
+    for byte in x.iter() {
+        accum += byte.count_ones() as u64;
+    }
+    accum
+}
+
+/// Computes hammming weight (naive version)
 /// Computes hamming distance
 /// Assumes x and y have same length
 /// slightly faster than naive version
-pub(crate) unsafe fn distance_faster(x: &[u8], y: &[u8]) -> u64 {
+pub fn distance_faster(x: &[u8], y: &[u8]) -> u64 {
     assert_eq!(x.len(), y.len());
     let mut accum: u64 = 0;
     let mut i: usize = 0;
@@ -22,8 +32,10 @@ pub(crate) unsafe fn distance_faster(x: &[u8], y: &[u8]) -> u64 {
         let x_u = x.as_ptr() as *const u64;
         let y_u = y.as_ptr() as *const u64;
         while 8 * i < x.len() - 8 {
-            let dist = (*x_u.add(i) ^ *y_u.add(i)).count_ones();
-            accum += dist as u64;
+            unsafe {
+                let dist = (*x_u.add(i) ^ *y_u.add(i)).count_ones();
+                accum += dist as u64;
+            }
             i += 1;
         }
     }
@@ -37,8 +49,9 @@ pub(crate) unsafe fn distance_faster(x: &[u8], y: &[u8]) -> u64 {
 }
 
 /// Computes hamming distance
-/// Assumes x and y have same memory alignemet
-/// Uses highly optimized avx2 versio if available
+/// Assumes x and y have same memory alignment
+/// Uses highly optimized avx2 version if available
+/// fallback on [distance_faster] if [x] and [y] have different alignment
 pub fn distance(x: &[u8], y: &[u8]) -> u64 {
     assert_eq!(x.len(), y.len());
     unsafe {
@@ -49,6 +62,17 @@ pub fn distance(x: &[u8], y: &[u8]) -> u64 {
     }
 }
 
+/// Computes hamming weight
+/// Uses highly optimized avx2 version if available
+pub fn weight(x: &[u8]) -> u64 {
+    unsafe {
+        if is_x86_feature_detected!("avx2") {
+            return lib_avx2::weight_vect(x);
+        }
+        weight_naive(x)
+    }
+}
+
 pub mod lib_avx2;
 
 #[cfg(test)]
